//
//	SCACT
//
//	Scene Controller for A Christmas Tableau
//
//	Code copyright (c) 2010 by Todd Krein (todd@krein.org)
//	All rights reserved, except as stated.
//
//	This program may be used by anyone for personal use, as long as
//	credit is given.
//
//	All source code and design files are availabel on my web site, or by mail.
//
//	Much of the code is "borrowed" from example code from TI or Maxim. 
//	Nothing in this statement changes the license under which that code is used.
//
//	N.B. 	This started out as a G2231 design, and outgrew it. It then migrated to the 
//			FE425, but had too many problems with the software I2C, and finally migrated
//			to the F4132. I appologize in advance for the bread crumbs left in the code.
//			I'll clean them up ASAP.  :-}
//

#include "msp430f4132.h"

#define	true	(1)
#define	false	(0)
#include "OneWireMaster.h"
#include "event.h"

//

void BoardPower(unsigned char);
unsigned char IsBoardPowered(void); 
void AppInit(void);					// configure the chip
void DelayMinute(short count);
void SleepUntil(unsigned short time);

//
//	All time in the system is handled in minutes since midnight,
//	so the maximum value is 24 * 60 = 1440. That makes life simple.
//

#define	turnOnTime ((12+5)*60)		// 5PM
#define	turnOffTime ((12+11)*60)	// 11PM

volatile unsigned short	gTimeOfDay;			// time of day in minutes
volatile unsigned char	gSec;				// seconds from the RTC
unsigned short	nextTime;					// time for next event
unsigned char	gNodeCount;					// how many 1-wire nodes are there?
struct EventStruct event;					// the active event

main() {
	
	char	cnt;
	unsigned char	rslt;	
	// 
	// Initialize the chip and
	// the application
	//
	AppInit();
	
	// turn on the board power, so we can scan
	BoardPower(true);
	
	//
	// Init the main subsystems
	//
	
	OneWireInit();
	
	//
	//	Count the number of slave nodes on the bus.
	//	Since the MSP430g2231 has a limited amount of RAM, we cannot
	//	keep a full table of nodes in memory. We'll remember the time for next event, 
	//	and we'll rescan at each opportunity.
	//
	//	N.B.,	now that we have a "real" MSP, we could store all of this in memory,
	//			but the scanning is pretty quick, and it works...
	//

	gNodeCount = OneWireNodeCount();	
	
	nextTime = turnOnTime;					// assume that the first event is power on
	
	//
	//	Loop forever, processing events
	
	while (1) {
		
		// see if we need to change the board 
		// power state
		
		if (IsBoardPowered() && (gTimeOfDay > turnOffTime)) {
			BoardPower(false);
		}
		else if (!IsBoardPowered() && (gTimeOfDay > turnOnTime) 
									&& (gTimeOfDay < turnOffTime)) {
			BoardPower(true);
		}
		
		// 
		// scan the rest of the tree, and find any event that needs to be done, 
		// as well as the next event
		//
		
		nextTime = 1441;		// later than possible
		
		rslt = OWFirst();		// look for the first one
		while (rslt) {
			while (1) {
				GetNodeEvent(&event);
				if (event.time > 0xfe00)			// marker for end of events
					break;
				
				if (event.time < gTimeOfDay)		// in the past
					continue;	
					
				if (event.time > gTimeOfDay) {		// in the future
					if (event.time < nextTime)		// in the future, but soonest
						nextTime = event.time;
					break;
				}
				
				if (event.time == gTimeOfDay)		// ah, the present
					ProcessEvent(cnt, &event);
			}		
				
			++cnt;				// found another one
			rslt = OWNext();	// look for more
		}
	
		//
		//	Done everything, wait until the next event time
		//
		
		if (nextTime == 1441)						// 	nothing more today
			nextTime = turnOnTime;					// wait until tomorrow
		
		//
		//	Processed this tick, back to sleep
		//
		
		SleepUntil(nextTime);						// wake up when we should
//		DelayMinute(1);
	}
	
}

//
//	AppInit
//
//	Set up all the timers, clocks, etc.
//
void AppInit(void) {
	WDTCTL = WDTPW +WDTHOLD;                  // Stop Watchdog Timer
	
	//1Mhz
//	BCSCTL1 = CALBC1_1MHZ;                    // Set range
//	DCOCTL = CALDCO_1MHZ;                     // Set DCO step + modulation */
// 	FLL_CTL0 |= XCAP14PF;                     // Configure load caps
	
	gTimeOfDay = 0;								// we start at the start
	gSec = 0;							// no seconds
	TACTL = TASSEL_1 + MC_2 + TAIE;           // ACLK, contmode, interrupt	  
		
}

// Timer_A2 Interrupt Vector (TAIV) handler
#pragma vector=TIMER0_A1_VECTOR
__interrupt void Timer_A(void)
{
	if (TAIV == 10) {
		// we should blink an LED
		gSec++;
		if (gSec == 60) {
			gTimeOfDay++;
			gSec = 0;
			if (gTimeOfDay == 1440)
				gTimeOfDay = 0;
		}
	}
}
//
//	BoardPower
//
//	Routine will control the power on the driver board going to the bus
//
//	!!! Stub for now, since the control FET isn't populated
//
void BoardPower(unsigned char turnOn) {
}

unsigned char IsBoardPowered(void) {
	return true;				// !!! until FET is populated, the board is always powered
}

void SleepUntil(unsigned short time) {
	
	while (gTimeOfDay < time)
		DelayMinute(1);
		
}

//
//	DelayMinute
//
//	This should put the CPU to sleep for some number of minutes, to save power,
//	but I'm having getting ti to wake up correctly. So, just delay for now.
//

void DelayMinute(short count) {
//	short mSec;
//	short sec;
	unsigned short startTime;
	
	startTime = gTimeOfDay;
	
//	while (count > 0) {
//		for (sec = 0; sec < 60; sec++) {
//			_BIS_SR(LPM3_bits + GIE);                 // Enter LPM3 w/ interrupt			
//			for (mSec = 0; mSec < 1000; mSec++) 
//				__delay_cycles (1000); // delay program execution for 1000 cycles 
//		}		
//	}

	while (gTimeOfDay < (startTime + count));
}
